""" Addon Information """
# ------------
# EventScripts Script-Addon:    - BackRaw's RoundStats (RoundStats, rs.py, 4.6kb)
# Author:                       - BackRaw - http://forums.eventscripts.com/profile.php?mode=viewprofile&u=13683
#                                           http://addons.eventscripts.com/addons/user/13683
# Version:                      - 1.1
# ------------

""" Imports """
# ------------
# EventScripts Core-Module      - ES functionality
# ------------
import es

""" Global Variables """
# ------------
# This player-dictionary will hold everyone's round-stats
# ------------
player = {}

""" Class - Player """
# ------------
# This class will manage all the players' names, kills, headshots, hits and damage
# ------------
class Player(object):
    def __init__(self, userid):
        self.userid = userid
        self.name = self.kills = self.hs = self.hits = self.damage = 0
    def hurt(self, damage):
        self.hits += 1
        self.damage += damage
    def death(self, hs):
        self.kills += 1
        if hs:
            self.hs += 1
    def reset(self):
        self.kills = self.hs = self.hits = self.damage = 0

""" Instance - getPlayer """
# ------------
# Rather call this instance instead of the class itself
# ------------
def getPlayer(userid):
    if es.exists('userid', userid):
        return Player(userid)
    raise ValueError, 'Userid %s does not exist!' % userid

""" Events """
# ------------
# Add the connected player to the player-dictionary
# ------------
def player_activate(ev):
    userid = ev['userid']
    player[userid] = getPlayer(userid)

# ------------
# Reset every the spawning player's name in the player-dictionary
# ------------
def player_spawn(ev):
    userid = ev['userid']
    if int(ev['es_userteam']) > 1 and userid in player:
        player[userid].name = ev['es_username']

# ------------
# Add one hit and the damage to the player's info
# ------------
def player_hurt(ev):
    attacker = ev['attacker']
    if int(attacker):
        player[attacker].hurt(int(ev['dmg_health']))

# ------------
# Add one kill to the player's info, if it's a headshot, add one headshot, too
# ------------
def player_death(ev):
    attacker = ev['attacker']
    if int(attacker):
        player[attacker].death(int(ev['headshot']))

# ------------
# Delete the player off the dictionary whenever he/she leaves
# ------------
def player_disconenct(ev):
    del player[ev['userid']]

# ------------
# Sort out "Most Kills", "Most Headshots" and "Most Damage" and announce it
# ------------
def round_end(ev):
    def sortOut(obj, what):
        obj.sort(cmp=None, key=None, reverse=True)
        win = 0
        for userid in player:
            if what == 'kills' and not obj.index(player[userid].kills) or what == 'damage' and not obj.index(player[userid].damage) or what == 'hs' and not obj.index(player[userid].hs):
                win = userid
        return win
    if int(ev['winner']) in (2, 3):
        kills = []
        damage = []
        hs = []
        for userid in player:
            kills.append(player[userid].kills)
            damage.append(player[userid].damage)
            hs.append(player[userid].hs)
        winkills = sortOut(kills, 'kills')
        windamage = sortOut(damage, 'damage')
        winhs = sortOut(hs, 'hs')
        kills = player[winkills].kills
        hs = player[winhs].hs
        damage = player[windamage].damage
        hits = player[windamage].hits
        if kills:
            es.msg('#multi', '#green[#lightgreenRoundStats#green] #defaultMost Kills: #green%s #lightgreen- #default%s' % (es.getplayername(winkills), kills))
        if hs:
            es.msg('#multi', '#green[#lightgreenRoundStats#green] #defaultMost Headshots: #green%s #lightgreen- #default%s' % (es.getplayername(winhs), hs))
        if damage and hits:
            es.msg('#multi', '#green[#lightgreenRoundStats#green] #defaultMost Damage: #green%s #lightgreen- #default%s (%s Hit%s)' % (es.getplayername(windamage), damage, hits, '' if hits == 1 else 's'))
        for userid in player:
            player[userid].reset()

""" EventScripts main-control functions """
# ------------
# Add every player to the player-dictionary and set the current version on load
# ------------
def load():
    es.ServerVar('roundstats_ver', '1.1').makepublic()
    for userid in es.getUseridList():
        userid = str(userid)
        player[userid] = getPlayer(userid)

# ------------
# Clean-up: clear the player-dictionary on load
# ------------
def unload():
    player.clear()

""" End Code """
